@@ -16,6 +16,7 @@
 
 
 import json
+import base64
 import posixpath
 import os
 import re
@@ -193,6 +194,38 @@ class Client:
             urljoin(self.url, f"/v2/admin/dockerRegistrySecrets/{name}")
         ).json()
 
+    def list_generic_secrets(self):
+        """
+        Lists registered generic secrets.
+        """
+        return self.sess.get(urljoin(self.url, f"/v2/admin/secrets")).json()[
+            "resources"
+        ]
+
+    def create_generic_secret(self, name, data):
+        """
+        Creates a generic secret.
+
+        :param name: The name of a secret.
+        :param data: Dict of ``key`` -> ``value``.
+        """
+        secret = {
+            "name": name,
+            "data": {
+                k: base64.b64encode(v.encode("ascii")).decode("ascii")
+                for k, v in data.items()
+            },
+        }
+        return self.sess.post(
+            urljoin(self.url, f"/v2/admin/secrets"), json=secret
+        ).json()
+
+    def delete_generic_secret(self, name):
+        """
+        Deletes a generic secret.
+        """
+        return self.sess.delete(urljoin(self.url, f"/v2/admin/secrets/{name}")).json()
+
     def list_git_repositories(self):
         """
         Lists registered git repositories.
@@ -375,7 +408,7 @@ class Client:
         """Starts a workflow execution.
 
         :param template: Template name or object returned from :meth:`Client.template` or :meth:`Client.list_templates`.
-        :param parameters: (optional) Dict of ``param-name``-> ``value``.
+        :param parameters: (optional) Dict of ``param-name`` -> ``value``.
         :param artifacts: (optional) Dict of ``binding-name`` -> ``S3 object URL or artifact or artifact ID``.
         """
 
@@ -438,7 +471,9 @@ class Client:
         logs = self.sess.get(
             urljoin(self.url, f"/v2/lm/executions/{getid(execution)}/logs")
         ).json()
-        return "".join([x["msg"] for x in logs["data"]["result"]])
+        return "\n".join(
+            [x["msg"] for x in logs["data"]["result"] if x["container"] == "main"]
+        )
 
     def create_artifact(self, url, kind, scenario, name, description=""):
         """Registers an artifact.
@@ -544,10 +579,13 @@ class Client:
 
         :param deployment: Deployment object or ID.
         """
-        d = self.stop_deployment(deployment)
-        self._wait_deployment_status(d, "STOPPED")
+        if isinstance(deployment, str):
+            deployment = self.deployment(deployment)
+        if deployment["status"] == "RUNNING":
+            self.stop_deployment(deployment)
+            self._wait_deployment_status(deployment, "STOPPED")
         return self.sess.delete(
-            urljoin(self.url, f"/v2/lm/deployments/{d['id']}")
+            urljoin(self.url, f"/v2/lm/deployments/{deployment['id']}")
         ).json()
 
     def predict(self, deployment, path="", data=None):
